@@ -256,11 +256,50 @@ class Agent(Insight, Channel, Catalog, Dataset):
             return None
 
 
-    def create_dataset(self, dataframe:pd.DataFrame, dataset_name:str, slug: str, settings: dict = None):
+    def _update_dataset_icon(self, dataset_id, icon_url):
+
+        url = self._base_url_askdata+"/smartdataset/datasets/"+dataset_id+"/settings"
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        logging.info("AUTH URL {}".format(url))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+
+        response = s.get(url=url, headers=headers)
+        response.raise_for_status()
+
+        settings = response.json()
+
+        settings["icon"] = icon_url
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        logging.info("AUTH URL {}".format(url))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+
+        response = s.put(url=url, json=settings, headers=headers)
+        response.raise_for_status()
+
+
+    def create_dataset(self, dataframe:pd.DataFrame, dataset_name:str, slug: str, icon_url=None, settings: dict = None):
 
         for col in dataframe.columns:
             if (dataframe[col].dtypes == "datetime64[ns]"):
-                dataframe[col] = dataframe[col].astype("string")
+                dataframe[col] = dataframe[col].astype("str")
 
         body = {"label": dataset_name, "rows": dataframe.to_dict(orient="record")}
 
@@ -288,11 +327,16 @@ class Agent(Insight, Channel, Catalog, Dataset):
         response = s.post(url=url, json=body, headers=headers)
         response.raise_for_status()
 
-    def update_dataset(self, dataframe: pd.DataFrame, dataset_name: str, slug: str, settings: dict = None):
+        dataset_id = response.json()["id"]
+
+        if(icon_url!=None):
+            self._update_dataset_icon(dataset_id, icon_url)
+
+    def update_dataset(self, dataframe: pd.DataFrame, dataset_name: str, slug: str, icon_url=None, settings: dict = None):
 
         for col in dataframe.columns:
             if (dataframe[col].dtypes == "datetime64[ns]"):
-                dataframe[col] = dataframe[col].astype("string")
+                dataframe[col] = dataframe[col].astype("str")
 
         body = {"label": dataset_name, "rows": dataframe.to_dict(orient="records")}
 
@@ -320,6 +364,10 @@ class Agent(Insight, Channel, Catalog, Dataset):
         response = s.put(url=url, json=body, headers=headers)
 
         response.raise_for_status()
+        dataset_id = response.json()["id"]
+
+        if(icon_url!=None):
+            self._update_dataset_icon(dataset_id, icon_url)
 
     '''La prima chiave è la stringa colonna del dataframe. Per ogni chiave ho dei setting:
     Name stringa
@@ -344,16 +392,16 @@ class Agent(Insight, Channel, Catalog, Dataset):
     '''
 
 
-    def create_or_replace_dataset(self, dataframe:pd.DataFrame, dataset_name:str, slug:str, settings:dict = None):
+    def create_or_replace_dataset(self, dataframe:pd.DataFrame, dataset_name:str, slug:str, icon_url=None, settings:dict = None):
         #Check if dataset exists
         dataset = self.get_dataset_by_slug(self._agentId, slug)
 
         if(dataset != None):
             #If it exists update it
-            self.update_dataset(dataframe, dataset_name, slug, settings)
+            self.update_dataset(dataframe, dataset_name, slug, icon_url, settings)
         else:
             # If not exists create a new one
-            self.create_dataset(dataframe, dataset_name, slug, settings)
+            self.create_dataset(dataframe, dataset_name, slug, icon_url, settings)
 
 
 
