@@ -23,9 +23,10 @@ class TimeDimensionGranularity(Enum):
 @dataclass
 class Field:
     column: str
-    aggregation: Optional[Union[str, SQLFunction, TimeDimensionGranularity]] = None
+    aggregation: Optional[Union[SQLFunction, TimeDimensionGranularity, str]] = None
     dataset: Optional[str] = None
     entityType: Optional[str] = None
+    alias: Optional[str] = None
 
 
 class SQLOperator(Enum):
@@ -59,13 +60,12 @@ class Sorting:
 
 @dataclass
 class Setting:
-    name: str
-    code: str
+    type: str
 
 
 @dataclass
 class Component:
-    type: str
+    component_type: str
     setting: Optional[List[Setting]] = None
 
 
@@ -88,13 +88,16 @@ class Query:
         fields_with_agg = []
         for field in self.fields:
             if field.aggregation is not None:
-                if isinstance(field.aggregation, str):
+                if isinstance(field.aggregation, SQLFunction):
+                    field_with_agg = field.aggregation.name + " ( " + field.column + " )"
+                elif isinstance(field.aggregation, TimeDimensionGranularity):
+                    field_with_agg = field.column
+                else:
                     field_with_agg = field.aggregation + " ( " + field.column + " )"
-                elif isinstance(field.aggregation, SQLFunction):
-                    field_with_agg = SQLFunction[field.aggregation] + " ( " + field.column + " )"
             else:
                 field_with_agg = field.column
-
+            if field.alias is not None:
+                field_with_agg += " AS " + field.alias
             fields_with_agg.append(field_with_agg)
 
         formatted_fields = ", ".join(fields_with_agg)
@@ -111,6 +114,7 @@ class Query:
         sql = sql.format(formatted_fields, table)
 
         where_conditions = []
+        having_conditions = []
         if self.where is not None:
             sql_where = " WHERE {}"
 
@@ -134,20 +138,23 @@ class Query:
                     elif operator == "EQ":
                         operator = "=="
                 if condition.field.aggregation is not None:
-                    if isinstance(condition.field.aggregation, str):
-                        field_with_agg = condition.field.aggregation + " ( " + condition.field.column + " )"
-                    elif isinstance(condition.field.aggregation, SQLFunction):
-                        field_with_agg = SQLFunction[condition.field.aggregation] + " ( " + condition.field.column + " )"
+                    having_conditions.append(condition)
+                    # if isinstance(condition.field.aggregation, SQLFunction):
+                    #     field_with_agg = condition.field.aggregation.name + " ( " + condition.field.column + " )"
+                    # elif isinstance(condition.field.aggregation, TimeDimensionGranularity):
+                    #     field_with_agg = condition.field.column
+                    # else:
+                    #     field_with_agg = condition.field.aggregation + " ( " + condition.field.column + " )"
                 else:
                     field_with_agg = condition.field.column
-                where_condition = (
-                        field_with_agg + " " + operator + " " + str(formatted_value)
-                )
-                where_conditions.append(where_condition)
-
-            formatted_where_conditions = " AND ".join(where_conditions)
-            sql_where = sql_where.format(formatted_where_conditions)
-            sql += sql_where
+                    where_condition = (
+                            field_with_agg + " " + operator + " " + str(formatted_value)
+                    )
+                    where_conditions.append(where_condition)
+            if where_conditions:
+                formatted_where_conditions = " AND ".join(where_conditions)
+                sql_where = sql_where.format(formatted_where_conditions)
+                sql += sql_where
 
         group_conditions = []
         sql_groupBy = " GROUP BY {}"
@@ -156,11 +163,54 @@ class Query:
             if field.entityType is not None:
                 if "P_DIMENSION" == field.entityType or "P_TIMEDIM" == field.entityType:
                     group_conditions.append(field.column)
-        if group_conditions != []:
+        if group_conditions:
             formatted_group = ", ".join(group_conditions)
             sql_groupBy = sql_groupBy.format(formatted_group)
             sql += sql_groupBy
 
+        if having_conditions:
+            if not group_conditions:
+                formatted_group = " '' "
+                sql_groupBy = sql_groupBy.format(formatted_group)
+                sql += sql_groupBy
+            sql_having = " HAVING {}"
+            all_having_conditions = []
+            for condition in having_conditions:
+                vars = []
+                for var in condition.value:
+                    var = str(var)
+                    if var.isnumeric():
+                        vars.append(var)
+                    else:
+                        vars.append(var)
+                formatted_value = "( " + ", ".join(vars) + " )"
+                if isinstance(condition.operator, str):
+                    operator = condition.operator
+                else:
+                    operator = condition.operator.name
+                    if operator == "GOE":
+                        operator = ">="
+                    elif operator == "LOE":
+                        operator = "<="
+                    elif operator == "EQ":
+                        operator = "=="
+                if condition.field.aggregation is not None:
+                    if isinstance(condition.field.aggregation, SQLFunction):
+                        field_with_agg = condition.field.aggregation.name + " ( " + condition.field.column + " )"
+                    elif isinstance(condition.field.aggregation, TimeDimensionGranularity):
+                        field_with_agg = condition.field.column
+                    else:
+                        field_with_agg = condition.field.aggregation + " ( " + condition.field.column + " )"
+                else:
+                    field_with_agg = condition.field.column
+                having_condition = (
+                            field_with_agg + " " + operator + " " + str(formatted_value)
+                    )
+                all_having_conditions.append(having_condition)
+            formatted_having_conditions = " AND ".join(all_having_conditions)
+            sql_having = sql_having.format(formatted_having_conditions)
+            sql += sql_having
+
         sorting_conditions = []
         if self.orderBy is not None:
             sql_orderby = " ORDER BY {}"
@@ -183,5 +233,5 @@ class Query:
 @dataclass
 class SmartQuery:
     queries: List[Query]
-    components: Optional[List[Component]] = None  # use this instead of chart
+    components: Optional[List[Component]] = None  # use this for of charts
     javascript: Optional[List[str]] = None
